@@ -630,7 +630,7 @@ class TestVectorsWycheproof(unittest.TestCase):
         
         try:
             cipher = AES.new(tv.key, AES.MODE_EAX, tv.iv, mac_len=tv.tag_size)
-        except ValueError, e:
+        except ValueError as e:
             assert len(tv.iv) == 0 and "Nonce cannot be empty" in str(e)
             return
 
@@ -646,7 +646,7 @@ class TestVectorsWycheproof(unittest.TestCase):
         
         try:
             cipher = AES.new(tv.key, AES.MODE_EAX, tv.iv, mac_len=tv.tag_size)
-        except ValueError, e:
+        except ValueError as e:
             assert len(tv.iv) == 0 and "Nonce cannot be empty" in str(e)
             return
 
